from enum import Enum
from typing import Tuple

class Color(str, Enum):
    """List of preset colors as defined in the Google Home app."""

    CoolWhite = "CoolWhite"
    Daylight = "Daylight"
    Ivory = "Ivory"
    WarmWhite = "WarmWhite"
    Incandescent = "Incandescent"
    Candlelight = "Candlelight"
    Snow = "Snow"
    GhostWhite = "GhostWhite"
    AliceBlue = "AliceBlue"
    LightGoldenrod = "LightGoldenrod"
    LemonChiffon = "LemonChiffon"
    AntiqueWhite = "AntiqueWhite"
    Gold = "Gold"
    Peru = "Peru"
    Chocolate = "Chocolate"
    SandyBrown = "SandyBrown"
    Coral = "Coral"
    Pumpkin = "Pumpkin"
    Tomato = "Tomato"
    Vermilion = "Vermilion"
    OrangeRed = "OrangeRed"
    Pink = "Pink"
    Crimson = "Crimson"
    DarkRed = "DarkRed"
    HotPink = "HotPink"
    Smitten = "Smitten"
    MediumPurple = "MediumPurple"
    BlueViolet = "BlueViolet"
    Indigo = "Indigo"
    LightSkyBlue = "LightSkyBlue"
    CornflowerBlue = "CornflowerBlue"
    Ultramarine = "Ultramarine"
    DeepSkyBlue = "DeepSkyBlue"
    Azure = "Azure"
    NavyBlue = "NavyBlue"
    LightTurquoise = "LightTurquoise"
    Aquamarine = "Aquamarine"
    Turquoise = "Turquoise"
    LightGreen = "LightGreen"
    Lime = "Lime"
    ForestGreen = "ForestGreen"

    def get_color_config(self) -> Tuple[int, int, int]:
        """Get the `hue`, `saturation`, and `color_temperature` of the color."""
